
# q-expansion of newform 4800.2.d.m, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4800
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 0, 2], 1], [[0, 4, 0, -2], 1], [[-2, 0, 4, 0], 1]]

hecke_ring_character_values = [[4351, [1, 0, 0, 0]], [901, [-1, 0, 0, 0]], [1601, [1, 0, 0, 0]], [577, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [-1, 0, 0, 0], [0, 0, 0, 0], [0, -1, 0, 0], [0, -1, 0, 1], [2, 0, -1, 0], [0, 0, 0, 1], [0, 0, 0, -2], [0, 2, 0, 0], [0, -1, 0, 2], [4, 0, 1, 0], [6, 0, -1, 0], [-2, 0, -2, 0], [0, 0, -2, 0], [0, 2, 0, 0], [-6, 0, -2, 0], [0, -3, 0, 1], [0, -2, 0, 0], [0, 0, 2, 0], [0, 0, 2, 0], [0, 5, 0, 0], [-4, 0, 3, 0], [8, 0, 0, 0], [6, 0, 2, 0], [0, 3, 0, -2], [0, 5, 0, 0], [0, 1, 0, 2], [0, 0, -2, 0], [0, 6, 0, 2], [0, 2, 0, -1], [0, -7, 0, -2], [0, -1, 0, -1], [0, -6, 0, -3], [0, -6, 0, 0], [0, -1, 0, 6], [-8, 0, 1, 0], [-18, 0, -1, 0], [4, 0, 0, 0], [0, 0, 0, 4], [18, 0, -2, 0], [0, 3, 0, 3], [0, -2, 0, 2], [8, 0, 0, 0], [0, -1, 0, 6], [10, 0, 4, 0], [8, 0, 5, 0], [0, -2, 0, 0], [0, 7, 0, 2], [20, 0, -2, 0], [0, -8, 0, 4], [0, 4, 0, -1], [16, 0, 0, 0], [26, 0, 0, 0], [0, -3, 0, -5], [0, 2, 0, 1], [0, 0, 0, 8], [0, 3, 0, 6], [8, 0, 1, 0], [-10, 0, 1, 0], [-6, 0, 2, 0], [16, 0, 2, 0], [-18, 0, 4, 0], [4, 0, -4, 0], [-16, 0, 4, 0], [0, 1, 0, -2], [-2, 0, 4, 0], [0, 8, 0, -2], [0, 5, 0, 4], [-4, 0, 2, 0], [0, 2, 0, 0], [0, 0, 0, 1], [-16, 0, -2, 0], [0, 1, 0, 0], [-2, 0, 3, 0], [0, -10, 0, 0], [0, 12, 0, 4], [0, 9, 0, 4], [26, 0, -3, 0], [22, 0, 0, 0], [-10, 0, 4, 0], [0, 1, 0, -5], [0, 8, 0, 0], [-24, 0, -2, 0], [0, 3, 0, -4], [16, 0, 5, 0], [4, 0, -10, 0], [10, 0, 8, 0], [0, 1, 0, 2], [0, -5, 0, -2], [0, 1, 0, 4], [24, 0, 4, 0], [-16, 0, -6, 0], [0, 1, 0, -6], [0, -7, 0, 5], [0, 8, 0, -2], [0, 2, 0, 0], [0, -15, 0, 4], [-30, 0, 0, 0], [-20, 0, 0, 0], [0, 2, 0, -2], [-16, 0, 2, 0], [6, 0, 6, 0], [12, 0, 2, 0], [-18, 0, -4, 0], [0, 4, 0, -2], [0, 11, 0, 2], [0, 0, 10, 0], [0, -10, 0, -5], [0, 0, -10, 0], [-6, 0, -12, 0], [0, -9, 0, 0], [2, 0, -7, 0], [0, -16, 0, 5], [0, 10, 0, 4], [-20, 0, 3, 0], [10, 0, 10, 0], [-12, 0, 8, 0], [0, -2, 0, 4], [6, 0, 4, 0], [0, 5, 0, -3], [0, -2, 0, 0], [0, -1, 0, -8], [-18, 0, 4, 0], [-32, 0, 4, 0], [0, 8, 0, -2], [0, 19, 0, -4], [0, 12, 0, -8], [24, 0, -6, 0], [0, 1, 0, 2], [-2, 0, 1, 0], [0, -4, 0, 10], [0, 4, 0, -8], [-36, 0, -3, 0], [-10, 0, -11, 0], [-30, 0, 0, 0], [-2, 0, 4, 0], [14, 0, 8, 0], [4, 0, -8, 0], [-2, 0, 6, 0], [2, 0, -6, 0], [0, -2, 0, -4], [0, -9, 0, -6], [0, -17, 0, -6], [4, 0, -12, 0], [0, 2, 0, -2], [-8, 0, -10, 0], [18, 0, 5, 0], [0, -4, 0, -13], [0, -6, 0, -8], [0, -12, 0, 0], [22, 0, -1, 0], [22, 0, -10, 0], [-36, 0, -4, 0], [0, 12, 0, -4], [8, 0, -2, 0], [0, 0, 8, 0], [-4, 0, -3, 0], [2, 0, -12, 0], [0, -7, 0, 4], [0, -3, 0, 0], [-12, 0, 4, 0], [0, -6, 0, -5], [0, -1, 0, 6], [0, -3, 0, 3], [0, -12, 0, -3], [0, 0, 0, 4], [32, 0, -5, 0], [-6, 0, 1, 0], [-22, 0, -4, 0], [-18, 0, 2, 0], [0, 1, 0, -7], [0, -8, 0, 2], [-8, 0, -4, 0], [0, 1, 0, 10], [-20, 0, 3, 0], [6, 0, -2, 0], [0, -2, 0, 4], [0, -3, 0, -10], [0, 7, 0, -8], [0, -12, 0, -6], [0, -1, 0, -10], [0, 15, 0, 9], [26, 0, 11, 0], [0, -10, 0, 7], [0, 4, 0, -4], [0, 1, 0, 0], [-42, 0, -3, 0], [-32, 0, -6, 0], [-14, 0, 0, 0], [-8, 0, -4, 0], [0, 7, 0, -6], [-60, 0, 0, 0], [0, -14, 0, 0], [0, 3, 0, -2], [0, 0, -14, 0], [0, 2, 0, -7], [22, 0, 4, 0], [26, 0, 3, 0], [0, 14, 0, 9], [0, 8, 0, -8], [0, 7, 0, -4], [20, 0, -5, 0], [2, 0, 11, 0], [10, 0, -8, 0], [0, 25, 0, 3], [30, 0, 0, 0], [-36, 0, -7, 0], [44, 0, 2, 0], [-18, 0, 10, 0], [0, 28, 0, 2], [0, 5, 0, -2], [0, 17, 0, 6], [0, 21, 0, 2], [4, 0, 12, 0], [32, 0, 2, 0], [-22, 0, -8, 0], [0, -3, 0, 0], [6, 0, 4, 0], [0, -12, 0, 4], [-22, 0, 6, 0], [0, -4, 0, 12], [16, 0, -3, 0], [-18, 0, 2, 0], [0, 5, 0, -16], [16, 0, 4, 0], [0, 4, 0, 6], [0, 6, 0, -7], [40, 0, -2, 0], [0, -11, 0, -4], [0, 3, 0, -5], [2, 0, 11, 0], [0, 14, 0, -12], [24, 0, -3, 0], [-30, 0, -2, 0], [28, 0, -12, 0], [0, -14, 0, -4], [-14, 0, 12, 0], [10, 0, 4, 0], [0, -15, 0, 7], [24, 0, 10, 0], [28, 0, -6, 0], [0, 8, 0, 6], [0, 19, 0, 2], [0, -6, 0, -4], [0, -20, 0, -3], [-48, 0, 2, 0], [0, 9, 0, 12], [0, 5, 0, -3], [0, 20, 0, -2], [0, -20, 0, 8], [-30, 0, 3, 0], [2, 0, -4, 0], [0, 18, 0, 4], [-6, 0, 4, 0], [22, 0, -2, 0], [0, -19, 0, 9], [0, 2, 0, -18], [40, 0, 2, 0], [14, 0, 6, 0], [0, 11, 0, 8], [0, -9, 0, 14], [12, 0, 8, 0], [0, 8, 0, 0], [46, 0, -5, 0], [0, 22, 0, 7], [0, -20, 0, -2], [0, -1, 0, 4], [30, 0, 8, 0], [44, 0, -8, 0], [18, 0, -8, 0], [0, 20, 0, 10], [-12, 0, -8, 0], [0, -7, 0, 10], [52, 0, 3, 0], [0, 5, 0, -6], [0, 1, 0, 6], [-60, 0, -2, 0], [0, 26, 0, 0], [-14, 0, -4, 0], [0, -5, 0, 7], [0, -18, 0, -4], [-16, 0, -13, 0], [0, -2, 0, 0], [0, 28, 0, 6], [0, 0, -10, 0], [0, 0, -4, 0], [0, 17, 0, 8], [46, 0, -8, 0], [-8, 0, -1, 0], [-18, 0, -4, 0], [0, -27, 0, 4], [8, 0, -14, 0], [0, 12, 0, 3], [0, 3, 0, -9], [14, 0, -13, 0], [0, 17, 0, 14], [-60, 0, 3, 0], [-14, 0, 2, 0], [0, -23, 0, -5], [-16, 0, -18, 0], [0, 5, 0, -6], [-62, 0, 4, 0], [24, 0, -11, 0], [16, 0, -10, 0], [0, 0, 0, -10], [0, 15, 0, -8], [-8, 0, -8, 0], [0, -8, 0, 20], [24, 0, 14, 0], [-10, 0, -17, 0], [0, 36, 0, 0], [0, -15, 0, -10], [6, 0, 4, 0], [72, 0, -2, 0], [0, 0, 0, -16], [18, 0, -12, 0], [0, 11, 0, -1], [8, 0, -2, 0], [0, 7, 0, -8], [30, 0, 0, 0], [0, -28, 0, -2], [0, -17, 0, -6], [0, -21, 0, 8], [0, -7, 0, -18], [0, 4, 0, 13], [-10, 0, -12, 0], [0, -2, 0, 8], [64, 0, -2, 0], [0, 16, 0, -12], [-10, 0, -18, 0], [0, -24, 0, 0], [-2, 0, 14, 0], [44, 0, 5, 0], [24, 0, 6, 0], [0, 22, 0, -8], [0, 0, 16, 0], [0, 8, 0, 14], [0, 36, 0, 1], [26, 0, 0, 0], [0, 9, 0, -10], [30, 0, -9, 0], [0, 28, 0, -1], [0, -9, 0, 8], [64, 0, -5, 0], [-26, 0, -14, 0], [0, -9, 0, -11], [0, 16, 0, -4], [52, 0, -12, 0], [0, 0, 2, 0], [-10, 0, -14, 0], [0, -2, 0, -4], [0, 17, 0, -6], [0, 7, 0, 0], [0, 11, 0, 10], [0, -6, 0, 8], [0, 12, 0, 3], [-64, 0, 8, 0], [0, -33, 0, -1], [0, 14, 0, -9], [0, 0, 0, -20], [-30, 0, 9, 0], [14, 0, -12, 0], [0, -8, 0, -16], [0, -23, 0, 3], [-8, 0, 10, 0], [0, -5, 0, -10], [-70, 0, -6, 0], [0, -31, 0, -4], [10, 0, -4, 0], [0, 9, 0, 15], [0, 2, 0, 4], [0, 12, 0, 8], [0, 17, 0, -2], [16, 0, 9, 0], [-14, 0, -9, 0], [0, -9, 0, -3], [-72, 0, 8, 0], [0, -11, 0, 12], [-6, 0, 20, 0], [0, 1, 0, 4], [0, 27, 0, 10], [0, 22, 0, -9], [0, -21, 0, -4], [0, 4, 0, -17], [0, 14, 0, 12], [0, -12, 0, 16], [-20, 0, -3, 0], [-42, 0, 9, 0], [52, 0, 8, 0], [0, -30, 0, -8], [-2, 0, 8, 0], [-42, 0, -10, 0], [0, 21, 0, 5], [20, 0, 16, 0], [8, 0, 20, 0], [0, -5, 0, -22], [56, 0, 3, 0], [34, 0, 14, 0], [0, 8, 0, 18], [0, 15, 0, -8], [0, 8, 0, 2], [0, -2, 0, -11], [0, 17, 0, 6], [0, 18, 0, 5], [0, 13, 0, 6], [-20, 0, 3, 0], [18, 0, 19, 0], [-10, 0, -16, 0], [-12, 0, 12, 0], [0, -27, 0, -1], [0, 5, 0, -2], [-2, 0, 0, 0], [-16, 0, -12, 0], [0, -24, 0, -10], [0, 19, 0, 8], [0, 17, 0, -14], [64, 0, 4, 0], [0, 11, 0, 0], [0, -22, 0, -17], [0, -22, 0, 0], [0, -15, 0, 2], [66, 0, 1, 0], [0, -12, 0, -12], [0, -45, 0, -3], [0, 29, 0, -6], [14, 0, -24, 0], [48, 0, 0, 0], [-14, 0, -4, 0], [0, -32, 0, -2], [56, 0, -2, 0]]
